#!/usr/bin/env python
#/* cgipm.py 
# * Copyright (C) 2000  Harry George
# *
# * This library is free software; you can redistribute it and/or
# * modify it under the terms of the GNU Library General Public
# * License as published by the Free Software Foundation; either
# * version 2 of the License, or (at your option) any later version.
# *
# * This library is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# * Library General Public License for more details.
# *
# * You should have received a copy of the GNU Library General Public
# * License along with this library; if not, write to the
# * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# * Boston, MA 02111-1307, USA.
# */


"""
Name:
    cgipm.py
    
Usage:
    Standalone (to test):
        cgipm.py [options] datapairs

    As library:
        import pyperl.cgipm
        query=cgipm.CGI()

Options:
    -h, --help        this message
    -v, --version     version of this program

Description:
    Provide perl CGI.pm-like syntax to python programmers
    Used to ease the migration from perl cgi's to python cgi's

Examples:
    As an import:
        import cgipm
        query=cgipm.CGI()
        ...
        query.header()
        query.start_html(title='My Title')
        ...
        query.startform(action='http://myurl.cgi')
        ...
        query.endform()
        query.endhtml()
        
    Debugging:
        mycgi.py name=xyz cost=100 


Revisions:
2000-02-21  Harry George   initial version
"""

import sys,os,getopt,time,re,string,types,exceptions,cgi
from cStringIO import StringIO
from string import join,split

modname="cgipm"
__version__="0.1"

def debug(ftn,txt):
    sys.stdout.write(modname+'.'+ftn+':'+txt+'\n')
    sys.stdout.flush()

def fatal(ftn,txt):
    sys.stdout.write(modname+'.'+ftn+':FATAL:'+txt+'\n')
    sys.stdout.flush()
    sys.exit(1)

def usage():
    print __doc__

#==============================
class CgiError(exceptions.Exception):
    def __init__(self,args):
        self.args=args
        

#===============================
#-----------------------
# Utilities
#-----------------------
def _moreargs(keywords,sep='=',quotes=0):
    if quotes:
        q='"'
    else:
        q=''

    s=StringIO()
    keys=keywords.keys()
    for key in keys:
        val=keywords[key]
        s.write(' '+key+sep+q+val+q)
    return s.getvalue()
#--------------------
color= \
{"black"   :"#000000" \
,"silver"  :"#C0C0C0" \
,"gray"    :"#808080" \
,"white"   :"#FFFFFF" \
,"maroon"  :"#800000" \
,"red"     :"#FF0000" \
,"purple"  :"#800080" \
,"fuchsia" :"#FF00FF" \
,"green"   :"#008000" \
,"lime"    :"#00FF00" \
,"olive"   :"#808000" \
,"yellow"  :"#FFFF00" \
,"navy"    :"#000080" \
,"blue"    :"#0000FF" \
,"teal"    :"#008080" \
,"aqua"    :"#00FFFF" \
}

#==============================
class CGI:
    #---class variables---
    _state_start        = 0
    _state_starthtml_ok = 1
    _state_endhtml_ok   = 2
    #--------------------------
    def __init__(self,*args):
        ftn="CGI"+".__init__"
        #---instance variables---
        self._params={}
        self.isquery=0
        self._state=CGI._state_start
        
        #---check for form response---
        env=os.environ
        method=None
        content_length=0
        if env.has_key('REQUEST_METHOD'):
            method=env['REQUEST_METHOD']
        if env.has_key('CONTENT_LENGTH'):
            content_length=env['CONTENT_LENGTH']
        if method=='POST' and content_length==0:
            self.die("invalid POST length")
        if method==None:
            #---fake it out---
            self.isquery=1
            self.get_debug_data(args)
        elif env.has_key('QUERY_STRING') and len(env['QUERY_STRING'])>0:
            self.isquery=1
            self.get_real_data()
        else:
            self.isquery=0
            

        #---simple shortcuts---
        self.p='<P>'
        self.end_p='</P>'
        self.hr='<HR>'

            
    #-----------------------
    # Basics
    #-----------------------
    def die(self,txt):
        msg='CGI.die:'+txt
        if self._state==CGI._state_start:
            self.header()
            self.start_html(title='CGI.die')
            print msg
            print self.end_html()
        elif self._state==CGI._state_starthtml_ok:
            print msg
            print self.end_html()
        elif self._state==CGI._state_endhtml_ok:
            print msg
        sys.exit(1)
            
    #-----------------------
    def get_debug_data(self,args):
        ftn="get_debug_data"
        def save_param(self,arg):
            #the self due to no scoping
            (name,value)=string.split(arg,'=')
            if self._params.has_key(name):
                oldvalue=self._params[name]
                value=oldvalue+','+value
            self._params[name]=value
            
        def load_args(self,save_param,args):
            #the self and save_param due to no scoping
            for arg in args:
                save_param(self,arg)
        
        #---any args?  else use comline---
        if not args:
            #---try using the commandline---
            args=sys.argv[1:]
            
        if args and len(args)==1:
            arg=args[0]
            #debug(ftn,"arg[0]="+`arg`)
            if isinstance(arg,types.StringType):
                if string.find(args[0],'=') > -1:
                    #---a singleton---
                    save_param(self,args[0])
                else:
                    #---a file name---
                    f=None
                    try:
                        f=open(arg,"r")
                    except IOError, e: self.die("cannot open initializer "+arg)
                    lines=f.readlines()
                    f.close()
                    load_args(self,save_param,lines)
            elif isinstance(arg,types.FileType):
                lines=f.readlines()
                f.close()
                load_args(self,lines)
            elif isinstance(arg,types.DictType):
                self._params=arg
            elif isinstance(arg,types.ListType):
                load_args(self,save_param,arg)
        elif args:
            #---a set of name=value pairs---
            for arg in args:
                save_param(self,arg)
        else:
            #---get from sdtin---
            print "Enter name=value lines, e.g."
            print 'mytext=abc def'
            print 'myint=123'
            print 'At the end, use ^D to stop'
            try:
                while 1:
                    arg=string.strip(sys.stdin.readline())
                    if not arg or arg=='':
                        break
                    save_param(self,arg)
            finally:
                pass
                
    #-----------------------
    def get_real_data(self):
        ftn="get_real_data"
        form=cgi.FieldStorage()
        for key in form.keys():
            username=""
            item=form[key]
            if type(item)==type([]):
                for i in range(len(item)):
                    if i>0: username=username+','
                    username=username+item[i].value
            else:
                username=item.value
            self._params[key]=username
        
    #-----------------------
    def header(self,type="text/html",**keywords):
        ftn="header"
        s=StringIO()
        s.write("Content-type:")
        if type:
            s.write(type)
        else:
            self.dir(ftn+" type required")
        if keywords:
            s.write(_moreargs(keywords,sep=': ',quotes=0))
        s.write('\n\n')
        return s.getvalue()
    #-----------------------
    def redirect(self,url):
        return 'HTTP/1.0 "301" '+url+'\n'
    #-----------------------
    def start_html(self,title,author=None,base=None,meta=None,style=None,
                   dtd=None,body=None,script=None):
        ftn="start_html"
        s=StringIO()
        s.write('<HTML>\n<HEAD>\n')
        if title:
            s.write('<TITLE>'+title+'</TITLE>\n')
        else:
            self.die(ftn+": title required")
        if author:
            s.write('<AUTHOR>'+author+'</AUTHOR>\n')
        s.write('<META http-equiv="Content-Type" \
                 content="text/html; charset=ISO-8859-1">')
        if base:
            s.write('<BASE>'+base+'</BASE>\n')
        if meta:
            keys=meta.keys()
            for key in keys:
                val=keys[key]
                s.write('<META NAME="%s" CONTENT="%s"' % (key,val))
        if style:
            s.write('<STYLE type="text/css">\n'+style+'\n</STYLE>')
        if script:
            if instanceof(script,types.StringType):
                s.write('<SCRIPT>\n'+script+'\n</SCRIPT>')
##          elif instanceof(script,types.DictType):
##                  s.write('<SCRIPT ')
##                  if script.has_key('language'):
##                      s.write(' LANGUAGE="%s"' % (script['language']))
##                  if script.has_key('src'):
##                      s.write(' SRC="%s"' % (script['src']))
##                  if script.has_key('code'): code=script['code']
##                      s.write(' SRC="%s"' % (script['src']))
                
                              

        s.write('\n</HEAD>')
        if body:
            s.write('\n<BODY '+body+'>')
        else:
            s.write('\n<BODY>')

        self.state=CGI._state_starthtml_ok
        return s.getvalue()
    #-----------------------
    def end_html(self):
        self.state=CGI._state_endhtml_ok
        return "</BODY></HTML>"

    #-----------------------
    def params(self):
        return self._params
    #-----------------------
    def param(self,key):
        if self._params.has_key(key):
            return self._params[key].value
        else:
            return None
    #------------------------
    # HTML Shortcuts
    #------------------------
    #------------------------
    def tag(self,tagname,*args,**other):
        """keywords go in the start tag, args go between start and end tags"""
        ftn="tag"
        s=StringIO()
        tag=string.upper(tagname)

        s.write(join(['<',tag],''))
        if other.has_key('other'):
            keyargs=other['other']
            for key in keyargs.keys():
                val=keyargs[key]
                if isinstance(val,types.StringType):                    
                    s.write(join([' ',key,'="',val,'"'],''))
                else:
                    s.write(join([' ',key,'=',str(val)],''))
        s.write('>')
        
        arglist=[]
        for arg in args:
            #debug(ftn,"arg="+`arg`)
            if   isinstance(arg,types.StringType):
                arglist.append(arg)
            elif isinstance(arg,types.TupleType):
                arglist.extend(list(arg))
            elif isinstance(arg,types.ListType):
                arglist.extend(arg)
            else:
                arglist.append(arg)
        #debug(ftn,"arglist="+`arglist`)
        s.write(join([join(arglist),'</',tag,'>'],''))
        return s.getvalue()
    #------------------------
    def start_tag(self,tagname,*args,**other):
        s=StringIO()
        tag=string.upper(tagname)
        arglist=[]
        for arg in args:
            if   isinstance(arg,types.TupleType):
                arglist.extend(list(arg))
            elif isinstance(arg,types.ListType):
                arglist.extend(arg)
            else:
                arglist.append(arg)
        s.write(join(['<',tag,' ',join(arglist)],''))
        for key in other.keys():
            val=other[key]
            if isinstance(val,types.StringType):                    
                s.write(join([' ',key,'="',val,'"'],''))
            elif isinstance(val,types.IntType):
                s.write(join([' ',key,'=',str(val)],''))
            else:
                pass
        s.write('>')
        return s.getvalue()
    #------------------------
    def end_tag(self,tagname):
        s=StringIO()
        tag=string.upper(tagname)
        s.write(join(['</',tag,'>'],''))
        return s.getvalue()

    #------------------------
    # HTML Shortcuts -- prebuilt
    #------------------------
    def h1(self,*txt):
        return self.tag('H1',txt)
    #------------------------
    def h2(self,*txt):
        return self.tag('H2',txt)
    #------------------------
    def h3(self,*txt):
        return self.tag('H3',txt)
    #------------------------
    def h4(self,*txt):
        return self.tag('H4',txt)
    #------------------------
    def h5(self,*txt):
        return self.tag('H5',txt)
    #------------------------
    def h6(self,*txt):
        return self.tag('H6',txt)
    #------------------------
    def ul(self,*txt,**other):
        return self.tag('UL',txt,other=other)
    #------------------------
    def ol(self,*txt,**other):
        return self.tag('OL',txt,other=other)
    #------------------------
    def li(self,*txt,**other):
        return self.tag('LI',txt,other=other)
    #------------------------
    def td(self,txt,**other):
        return self.tag('TD',txt,other=other)
    #------------------------
    def th(self,txt,**other):
        return self.tag('TH',txt,other=other)
    #------------------------
    def tr(self,*txt,**other):
        s=StringIO()
        s.write(self.start_tag('TR',other=other))
        for t in txt:
            s.write(t)
        s.write(self.end_tag('TR'))
        return s.getvalue()
               
    #-----------------------
    # Forms
    #-----------------------
    def startform(self,action=None,method=None,enctype=None):
        ftn="startform"
        s=StringIO()
        s.write("<FORM")
        if action:
            s.write(' action="'+action+'"')
        else:
            raise CgiError, ftn+": action required"
        if method:
            s.write(' method="'+method+'"')
        else:
            raise CgiError, ftn+": method required"
        if enctype:
            s.write(' enctype="'+entype+'"')
        s.write('>\n')
        return s.getvalue()
    #-----------------------
    def endform(self):
        return '\n</FORM>\n'
    #-----------------------
    def input(self,name,type='text',value='',other=None):
        s=StringIO()
        s.write('<INPUT type="%s" name="%s" value="%s"' % (type,name,value))
        if other and len(other) > 0:
            keys=other.keys()
            for key in keys:
                val=other[key]
                if isinstance(val,types.StringType):                    
                    s.write(' '+key+'="'+val+'"')
                else:
                    s.write(' '+key+'='+str(val))
        s.write('>')
        return s.getvalue()
    #-----------------------
    def textfield(self,name,value='',**other):
        return self.input(name,type='text',value=value,other=other)
    #-----------------------
    def password(self,name,value='',**other):
        return self.input(name,type='password',value=value,other=other)
    #-----------------------
    def hidden(self,name,value='',**other):
        return self.input(name,type='hidden',value=value,other=other)
    #-----------------------
    def button(self,name,value='',**other):
        return self.input(name,type='button',other=other)
##  #-----------------------
##  def checkbox(self,name,value='',**other):
##      return self.input(name,type='checkbox',value=value,other=other)
##  #-----------------------
##  def radio(self,name,value='',**other):
##      return self.input(name,type='radio',value=value,other=other)
##  #-----------------------
    def submit(self,label,value='',**other):
        return self.input(name=label,type='submit',value=value,other=other)     
    #-----------------------
    def reset(self,label,value='',**other):
        return self.input(name=label,type='reset',value=value,other=other)      
    #-----------------------
    def button(self,label,value='',**other):
        return self.input(name=label,type='button',value=value,other=other)
    #-----------------------
    def image_button(self,label,value='',**other):
        return self.input(name=label,type='image',value=value,other=other)
    #-----------------------
    def checkbox_group(self,name,values,defaults=None,linebreak=0,labels=None,**other):
        s=StringIO()
        #---find the selected items, if any---
        selected={}
        if defaults:
            if isinstance(defaults,types.IntType):
                selected[defaults]=1
            elif isinstance(defaults,types.ListType):
                for d in defaults:
                    selected[d]=1
        #---build the group---
        for value in values:
            s.write('<INPUT type="checkbox" name="%s" value="%s"' % (name,value))
            if selected.has_key(value):
                s.write(' checked')
            s.write('> ')
            if labels and labels.has_key(value):
                s.write(labels[value])
            else:
                s.write(value)
            if linebreak:
                s.write('<BR>')
            s.write('\n')
        return s.getvalue()
    #-----------------------
    def radio_group(self,name,values,default=None,linebreak=0,labels=None,**other):
        s=StringIO()
        #---find the selected items, if any---
        selected={}
        if default:
            if default=='-':
                pass
            else:
                selected[default]=1
        #---build the group---
        for value in values:
            s.write('<INPUT type="radio" name="%s" value="%s"' % (name,value))
            if selected.has_key(value):
                s.write(' checked')
            s.write('> ')
            if labels and labels.has_key(value):
                s.write(labels[value])
            else:
                s.write(value)
            if linebreak:
                s.write('<BR>')
            s.write('\n')
        return s.getvalue()
    #-----------------------
    def popup_menu(self,name,values,default=None,labels=None,**other):
        s=StringIO()
        #---find the selected items, if any---
        selected={}
        if default:
            if default=='-':
                pass
            else:
                selected[default]=1
        #---build the group---
        s.write('<SELECT name="%s" ' % (name))
        s.write('>\n')
        for value in values:
            s.write('<OPTION value="%s"' % (value))
            if selected.has_key(value):
                s.write(' selected')
            s.write('> ')
            if labels and labels.has_key(value):
                s.write(labels[value])
            else:
                s.write(value)
            s.write('\n')
        s.write('</SELECT>')
        return s.getvalue()
    #-----------------------
    def scrolling_list(self,name,values,default=None,size=None,multiple=0,labels=None,**other):
        s=StringIO()
        #---find the selected items, if any---
        selected={}
        if default:
            if default=='-':
                pass
            else:
                selected[default]=1
        #---build the group---
        s.write('<SELECT name="%s" ' % (name))
        if size:
            s.write(' size=%d' % size)
        if multiple:
            s.write(' multiple')            
        s.write('>\n')
        for value in values:
            s.write('<OPTION value="%s"' % (value))
            if selected.has_key(value):
                s.write(' selected')
            s.write('> ')
            if labels and labels.has_key(value):
                s.write(labels[value])
            else:
                s.write(value)
            s.write('\n')
        s.write('</SELECT>')
        return s.getvalue()
    #-----------------------
    def textarea(self,name,default=None,rows=1,cols=20,
                 current='',**keywords):
        s=StringIO()
        s.write('<TEXTAREA name="%s"' % (name))
        if default: s.write(' default="%s"' % (default))
        if rows:
            s.write(' rows="%d"' % (rows))
        else:
            raise CgiError, ftn+":missing rows"
        if cols:
            s.write(' cols="%d"' % (cols))
        else:
            raise CgiError, ftn+":missing cols"
        s.write('>')
        if current: s.write(current)
        s.write('\n</TEXTAREA>\n')
        return s.getvalue()
    
#---main-----------
#exemplars; tweak as needed
opt_a=0     #boolean arg, default is false
opt_b=None  #string arg, default is undefined

#positional args, default is empty
pargs=[]    

if __name__ == '__main__':
    ftn = "main"
    opts,pargs=getopt.getopt(sys.argv[1:],'hvab:',
                 ['help','version','aa','bb='])
    for opt in opts:
        if opt[0]=='-h' or opt[0]=='--help':
            usage()
            sys.exit(0)
        elif opt[0]=='-v' or opt[0]=='--version':
            print modname+": version="+version
        elif opt[0]=='-a' or opt[0]=='--aa':
            opt_a=1
        elif opt[0]=='-b' or opt[0]=='--bb':
            opt_b=opt[1]

    #---make the object and run it---
    x=CGI()
   
